/*--------------------------------------------------------------------------
*    This file is part of InversionKit.
*
*    InversionKit is free software: you can redistribute it and/or modify
*    it under the terms of the GNU General Public License as published by
*    the Free Software Foundation, either version 3 of the License, or
*    (at your option) any later version.
*
*    InversionKit is distributed in the hope that it will be useful,
*    but WITHOUT ANY WARRANTY; without even the implied warranty of
*    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*    GNU General Public License for more details.
*
*    You should have received a copy of the GNU General Public License
*    along with InversionKit.  If not, see <http://www.gnu.org/licenses/>.
*
****************************************************************************
*
*    Copyright (c) Daniel Reese, Sergei Zharkov, 2008
*
****************************************************************************/
/////////////////////////////////////////////////////////////////////////////
/*
 * @(#)OptionPaneDemo.java      1.9 04/07/26
 *
 * Copyright (c) 2004 Sun Microsystems, Inc. All Rights Reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * -Redistribution of source code must retain the above copyright notice, this
 *  list of conditions and the following disclaimer.
 *
 * -Redistribution in binary form must reproduce the above copyright notice,
 *  this list of conditions and the following disclaimer in the documentation
 *  and/or other materials provided with the distribution.
 *
 * Neither the name of Sun Microsystems, Inc. or the names of contributors may
 * be used to endorse or promote products derived from this software without
 * specific prior written permission.
 *
 * This software is provided "AS IS," without a warranty of any kind. ALL
 * EXPRESS OR IMPLIED CONDITIONS, REPRESENTATIONS AND WARRANTIES, INCLUDING
 * ANY IMPLIED WARRANTY OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE
 * OR NON-INFRINGEMENT, ARE HEREBY EXCLUDED. SUN MIDROSYSTEMS, INC. ("SUN")
 * AND ITS LICENSORS SHALL NOT BE LIABLE FOR ANY DAMAGES SUFFERED BY LICENSEE
 * AS A RESULT OF USING, MODIFYING OR DISTRIBUTING THIS SOFTWARE OR ITS
 * DERIVATIVES. IN NO EVENT WILL SUN OR ITS LICENSORS BE LIABLE FOR ANY LOST
 * REVENUE, PROFIT OR DATA, OR FOR DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL,
 * INCIDENTAL OR PUNITIVE DAMAGES, HOWEVER CAUSED AND REGARDLESS OF THE THEORY
 * OF LIABILITY, ARISING OUT OF THE USE OF OR INABILITY TO USE THIS SOFTWARE,
 * EVEN IF SUN HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
 *
 * You acknowledge that this software is not designed, licensed or intended
 * for use in the design, construction, operation or maintenance of any
 * nuclear facility.
 */

/*
 * @(#)OptionPaneDemo.java      1.9 04/07/26
 */

/////////////////////////////////////////////////////////////////////////////
package graph;

import graph.Plot;
import java.io.File;
import javax.swing.JButton;
import javax.swing.JComponent;
import javax.swing.JPanel;
import javax.swing.JFrame;
import javax.swing.JOptionPane;
import javax.swing.JFileChooser;
import javax.swing.BoxLayout;
import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.Color;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.event.MouseMotionListener;
import java.lang.Cloneable;

/////////////////////////////////////////////////////////////////////////////
/**
*
* The purpose of this class is to create interactive plots in Java.
*
*/
/////////////////////////////////////////////////////////////////////////////

public class IPlot extends JComponent implements Cloneable
{
  // This panel contains a number buttons to be used with the plot:
  private JPanel buttonBox = new JPanel();
  private JButton IDLButton;
  private JButton TXTButton;
  private JButton DuplicateButton;
  private JButton AutoScaleButton;
  private boolean IOenabled = true;
  
  // This contains the plot itself:
  private Plot graphicsArea;

  /////////////////////////////////////////////////////////////////////////////
  // This sets up the interactive plot.  I'm assuming the actual plot has
  // already been initialised.
  /////////////////////////////////////////////////////////////////////////////
  private void initIPlot() {
    initOutputButtons();
    initAutoScaleButton();
    initGraphicsArea();
    initDuplicateButton();

    buttonBox.setLayout(new BoxLayout(buttonBox, BoxLayout.Y_AXIS));
    buttonBox.add(IDLButton);
    buttonBox.add(TXTButton);
    buttonBox.add(AutoScaleButton);
    buttonBox.add(DuplicateButton);
    setLayout(new BorderLayout());
    add("Center", graphicsArea);
    add("West", buttonBox);
  }

  /////////////////////////////////////////////////////////////////////////////
  /** This enables or disables all input/output related buttons, depending on
  *   the value of flag. */
  /////////////////////////////////////////////////////////////////////////////
  public void setIOenabled(boolean flag) {
    IDLButton.setEnabled(flag);
    TXTButton.setEnabled(flag);
    IOenabled = flag;
  }

  /////////////////////////////////////////////////////////////////////////////
  // Enable the graphics area to respond to mouse events adding a mouse
  // listener and mouse motion listener.
  /////////////////////////////////////////////////////////////////////////////
  private void initGraphicsArea() {
    MyMouseListener l = new MyMouseListener();
    graphicsArea.addMouseListener(l);
    graphicsArea.addMouseMotionListener(l);
  }

  /////////////////////////////////////////////////////////////////////////////
  // Private class which describes the mouse listener/mouse motion listener
  // that goes with graphicsArea.
  /////////////////////////////////////////////////////////////////////////////
  private class MyMouseListener implements MouseListener, MouseMotionListener {

    // This keeps track of whether the mouse is
    // being dragged (and has stayed within
    // graphicsArea):
    private boolean drag = false;
    
    // This keeps track of whether or not there
    // is a previous rectangle that needs to be
    // erased.
    private boolean prevRect = false;

    // (x0,y0) = start of drag
    // (x1,y1) = end of drag
    private int x0, y0, x1, y1;

    /** Unused method which still needs to be
    *   implemented. */
    public void mouseClicked(MouseEvent e) {
    }

    /** Unused method which still needs to be
    *   implemented. */
    public void mouseEntered(MouseEvent e) {
    }

    /** Unused method which still needs to be
    *   implemented. */
    public void mouseMoved(MouseEvent e) {
    }

    public void mouseDragged(MouseEvent e) {
      if (drag) {
        if (prevRect) {
          // this erases the previous rectangle:
          graphicsArea.drawRect(x0,y0,x1,y1);
        }
        prevRect = true;
    
        // this draws the new rectangle:
        x1 = e.getX();
        y1 = e.getY();
        graphicsArea.drawRect(x0,y0,x1,y1);
      }
    }

    public void mouseExited(MouseEvent e) {
      if (drag&&prevRect) 
        graphicsArea.drawRect(x0,y0,x1,y1);
      drag = false;
      prevRect = false;
    }

    public void mousePressed(MouseEvent e) {
      drag = true;
      prevRect = false;
      x0 = e.getX();
      y0 = e.getY();
    }

    public void mouseReleased(MouseEvent e) {

      // To avoid inverting the range, we put the
      // appropriate values in the following
      // variables and use them instead:
      int xleft, xright, ytop, ybottom;

      if (drag) {
        x1 = e.getX();
        y1 = e.getY();
        if (x0==x1) return;
        if (y0==y1) return;
        xleft=x0<x1?x0:x1;
        xright=x0<x1?x1:x0;
        ytop=y0<y1?y0:y1;
        ybottom=y0<y1?y1:y0;
        graphicsArea.zoom(xleft,xright,ybottom,ytop);
        graphicsArea.repaint();
      }
      drag = false;
      prevRect = false;
    }
  }

  /////////////////////////////////////////////////////////////////////////////
  // Initialise the autoscale button:
  /////////////////////////////////////////////////////////////////////////////
  private void initAutoScaleButton() {
    AutoScaleButton = createButton("AS");
    AutoScaleButton.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        graphicsArea.findRange();
        repaint();
      }
    });
  }

  /////////////////////////////////////////////////////////////////////////////
  // Initialise the IDL button:
  /////////////////////////////////////////////////////////////////////////////
  private void initOutputButtons() {
    IDLButton = createButton("IDL");
    TXTButton = createButton("TXT");
    ActionListener l = new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        JButton source = (JButton)e.getSource();
        File aFile;
        int result;

        // open in current directory:
        JFileChooser fc = new JFileChooser(".");

        // show the filechooser
        result = fc.showSaveDialog(IDLButton);  // put parent component

        // get filename
        if(result == JFileChooser.APPROVE_OPTION) {
           aFile = fc.getSelectedFile();

          // if file exists, check before overwriting:
          if (aFile.exists()) {
            result = JOptionPane.showConfirmDialog(IDLButton,
                 "Are you sure you want to overwrite "+aFile.getName()+"?",
                 "File exists",JOptionPane.YES_NO_OPTION);
          } else {
            result = JOptionPane.YES_OPTION;
          }
        
          if(result == JOptionPane.YES_OPTION) {
            if (source == IDLButton) {
              graphicsArea.writeIDLfile(aFile,source);
            } else {
              graphicsArea.writeTXTfile(aFile,source);
            }
          }
        }
      }
    };
    IDLButton.addActionListener(l);
    TXTButton.addActionListener(l);
  }

  /////////////////////////////////////////////////////////////////////////////
  /** This initialises the DuplicateButton, which produces a duplicate of the
  *   plot in a separate window. */
  /////////////////////////////////////////////////////////////////////////////
  private void initDuplicateButton() {
    DuplicateButton = createButton("DUPL");
    DuplicateButton.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        JFrame f = new JFrame(graphicsArea.title);
        IPlot newIPlot = new IPlot(graphicsArea.clone());
        f.add("Center", newIPlot);
        f.setSize(500,400);
        newIPlot.initIPlot();
        newIPlot.setIOenabled(IOenabled);
        newIPlot.repaint();
        f.setVisible(true);
      }
    });
  }

  /////////////////////////////////////////////////////////////////////////////
  /** This clones an IPlot. */
  /////////////////////////////////////////////////////////////////////////////
  public IPlot clone() {
    return new IPlot(this.graphicsArea.clone());
  }

  /////////////////////////////////////////////////////////////////////////////
  /** A constructor which creates an IPlot from a Plot. */
  /////////////////////////////////////////////////////////////////////////////
  public IPlot(Plot aPlot) {
    this.graphicsArea = aPlot;
  }

    
  /////////////////////////////////////////////////////////////////////////////
  /**
   * A constructor.  If the arrays do not have the same lengths,
   * the longer one is truncated, but an internal error is issued.
   *
   * x      = array of x values
   * y      = array of y values
   * ls     = integer giving the line style
   * ps     = integer giving the choice of symbols
   * aColor = a color
   */
  /////////////////////////////////////////////////////////////////////////////
  public IPlot(double[] x, double[] y, int ls, int ps, Color aColor) {
    graphicsArea = new Plot(x,y,ls,ps,aColor);
    initIPlot();
  }

  /////////////////////////////////////////////////////////////////////////////
  /** Another constructor.  Input x[], y[], linestyle, and choice of symbols.*/
  /////////////////////////////////////////////////////////////////////////////
  public IPlot(double[] x, double[] y, int ls, int ps) {
  
    // Note: it is best to use the correct Plot constructor, rather than
    // doing "this(x,y,ls,ps,Color.black)" because this way any modifications
    // to Plot(x,y,ls,ps) is then taken into account.

    graphicsArea = new Plot(x,y,ls,ps);
    initIPlot();
  }

  /////////////////////////////////////////////////////////////////////////////
  /** Another constructor.  Input x[], y[], linestyle, and choice of symbols.*/
  /////////////////////////////////////////////////////////////////////////////
  public IPlot(double[] x, double[] y, int ls, Color aColor) {
    graphicsArea = new Plot(x,y,ls,aColor);
    initIPlot();
  }

  /////////////////////////////////////////////////////////////////////////////
  /** Another constructor.  Input x[], y[] and linestyle.  */
  /////////////////////////////////////////////////////////////////////////////
  public IPlot(double[] x, double[] y, int ls) {
    graphicsArea = new Plot(x,y,ls);
    initIPlot();
  }

  /////////////////////////////////////////////////////////////////////////////
  /** Another constructor.  Input x[], y[].  */
  /////////////////////////////////////////////////////////////////////////////
  public IPlot(double[] x, double[] y) {
    graphicsArea = new Plot(x,y);
    initIPlot();
  }
  
  /////////////////////////////////////////////////////////////////////////////
  /** Another constructor. Input title, xtitle, ytitle and number of data sets.*/
  /////////////////////////////////////////////////////////////////////////////
  public IPlot(String title, String xtitle, String ytitle, int nDataSets) {
    graphicsArea = new Plot(title,xtitle,ytitle,nDataSets);
    initIPlot();
  }

  /////////////////////////////////////////////////////////////////////////////
  /** Another constructor.  Input title, xtitle and ytitle.  */
  /////////////////////////////////////////////////////////////////////////////
  public IPlot(String title, String xtitle, String ytitle) {
    graphicsArea = new Plot(title,xtitle,ytitle);
    initIPlot();
  }

  /////////////////////////////////////////////////////////////////////////////
  /** This allows the titles to be modified */
  /////////////////////////////////////////////////////////////////////////////
  public void setTitles(String title, String xtitle, String ytitle) {
    graphicsArea.setTitles(title,xtitle,ytitle);
  }

  /////////////////////////////////////////////////////////////////////////////
  /** This adds a data set (x,y) with a specified linestyle, choice of 
  *   symbols and color. */
  /////////////////////////////////////////////////////////////////////////////
  public void addDataSet(double[] x, double[] y, int ls, int ps, Color aColor) {
    graphicsArea.addDataSet(x,y,ls,ps,aColor);
  }

  /////////////////////////////////////////////////////////////////////////////
  /** This adds a data set (x,y) with a specified linestyle and choice of symbols. */
  /////////////////////////////////////////////////////////////////////////////
  public void addDataSet(double[] x, double[] y, int ls, int ps) {
    graphicsArea.addDataSet(x,y,ls,ps);
  }

  /////////////////////////////////////////////////////////////////////////////
  /** This adds a data set (x,y) with a specified linestyle and choice of symbols. */
  /////////////////////////////////////////////////////////////////////////////
  public void addDataSet(double[] x, double[] y, int ls, Color aColor) {
    graphicsArea.addDataSet(x,y,ls,aColor);
  }

  /////////////////////////////////////////////////////////////////////////////
  /** This adds a data set (x,y) with a specified linestyle. */
  /////////////////////////////////////////////////////////////////////////////
  public void addDataSet(double[] x, double[] y, int ls) {
    graphicsArea.addDataSet(x,y,ls);
  }
  
  /////////////////////////////////////////////////////////////////////////////
  /** This adds a data set (x,y). */
  /////////////////////////////////////////////////////////////////////////////
  public void addDataSet(double[] x, double[] y) {
    graphicsArea.addDataSet(x,y);
  }
  
  /////////////////////////////////////////////////////////////////////////////
  /**
   * This removes data set number ds from arrays x and y, where:
   *    0 <= ds < nDataSets
   */
  /////////////////////////////////////////////////////////////////////////////
  public void removeDataSet(int ds) {
    graphicsArea.removeDataSet(ds);
  }
  
  /////////////////////////////////////////////////////////////////////////////
  /**
   * This replaces data set number ds using arrays x and y, linestyle ls
   * and symbol choice ps.
   *
   * ds     = data set that will be replaced (0 <= ds < nDataSets)
   * x      = new array of x values
   * y      = new array of y values
   * ls     = new linestyle
   * ps     = new choice of symbols
   * aColor = new choice of color
   */
  /////////////////////////////////////////////////////////////////////////////
  public void replaceDataSet(int ds, double[] x, double[] y, int ls,
                             int ps, Color aColor) {
    graphicsArea.replaceDataSet(ds,x,y,ls,ps,aColor);
  }
  
  /////////////////////////////////////////////////////////////////////////////
  /**
   * This replaces data set number ds using arrays x and y, linestyle ls
   * and symbol choice ps.
   *
   * ds     = data set that will be replaced (0 <= ds < nDataSets)
   * x      = new array of x values
   * y      = new array of y values
   * ls     = new linestyle
   * aColor = new choice of color
   */
  /////////////////////////////////////////////////////////////////////////////
  public void replaceDataSet(int ds, double[] x, double[] y, int ls,
                             Color aColor) {
    graphicsArea.replaceDataSet(ds,x,y,ls,aColor);
  }
  
  /////////////////////////////////////////////////////////////////////////////
  /**
   * This replaces data set number ds using arrays x and y, linestyle ls
   * and symbol choice ps.
   *
   * ds = data set that will be replaced (0 <= ds < nDataSets)
   * x  = new array of x values
   * y  = new array of y values
   * ls = new linestyle
   * ps = new choice of symbols
   */
  /////////////////////////////////////////////////////////////////////////////
  public void replaceDataSet(int ds, double[] x, double[] y, int ls, int ps) {
    graphicsArea.replaceDataSet(ds,x,y,ls,ps);
  }
  
  /////////////////////////////////////////////////////////////////////////////
  /**
   * This replaces data set number ds using arrays x and y, linestyle ls
   * and symbol choice ps.
   *
   * ds  = data set that will be replaced (0 <= ds < nDataSets)
   * x   = new array of x values
   * y   = new array of y values
   * ls  = new linestyle
   */
  /////////////////////////////////////////////////////////////////////////////
  public void replaceDataSet(int ds, double[] x, double[] y, int ls) {
    graphicsArea.replaceDataSet(ds,x,y,ls);
  }

  /////////////////////////////////////////////////////////////////////////////
  /**
   * This replaces data set number ds using arrays x and y, linestyle ls
   * and symbol choice ps.
   *
   * ds  = data set that will be replaced (0 <= ds < nDataSets)
   * x   = new array of x values
   * y   = new array of y values
   */
  /////////////////////////////////////////////////////////////////////////////
  public void replaceDataSet(int ds, double[] x, double[] y) {
    graphicsArea.replaceDataSet(ds,x,y);
  }
 
  /////////////////////////////////////////////////////////////////////////////
  /** This finds appropriate values for xmin, xmax, ymin and ymax. */
  /////////////////////////////////////////////////////////////////////////////
  public void findRange() {
    graphicsArea.findRange();
  }

  /////////////////////////////////////////////////////////////////////////////
  /** This finds appropriate values for xmin, xmax, ymin and ymax. */
  /////////////////////////////////////////////////////////////////////////////
  public void findRange(int[] selection) {
    graphicsArea.findRange(selection);
  }

  /////////////////////////////////////////////////////////////////////////////
  /** This allows the user to set xmin */
  /////////////////////////////////////////////////////////////////////////////
  public void setXmin(double xmin) {graphicsArea.setXmin(xmin);}

  /////////////////////////////////////////////////////////////////////////////
  /** This allows the user to set xmax */
  /////////////////////////////////////////////////////////////////////////////
  public void setXmax(double xmax) {graphicsArea.setXmax(xmax);}

  /////////////////////////////////////////////////////////////////////////////
  /** This allows the user to set ymin */
  /////////////////////////////////////////////////////////////////////////////
  public void setYmin(double ymin) {graphicsArea.setYmin(ymin);}

  /////////////////////////////////////////////////////////////////////////////
  /** This allows the user to set ymax */
  /////////////////////////////////////////////////////////////////////////////
  public void setYmax(double ymax) {graphicsArea.setYmax(ymax);}

  /////////////////////////////////////////////////////////////////////////////
  /** This finds appropriate values for xmin, xmax, ymin and ymax. */
  /////////////////////////////////////////////////////////////////////////////
  public void repaint() {
    graphicsArea.repaint();
  }

  /////////////////////////////////////////////////////////////////////////////
  // This creates buttons of the right size for IPlot.  I don't know how
  // it works, but simply copied it from JDK SwingSet2 demo program.
  // I modified it to include s as a button title.
  /////////////////////////////////////////////////////////////////////////////
  private JButton createButton(String s) {
    JButton b = new JButton(s) {
      public Dimension getMaximumSize() {
        int width = Short.MAX_VALUE;
        int height = super.getMaximumSize().height;
        return new Dimension(width, height);
      }
    };
    return b;
  }

  /////////////////////////////////////////////////////////////////////////////
  /** This nullifies a plot by setting all the arrays contained in
  *   graphicsArea.x and graphicsArea.y to null */
  /////////////////////////////////////////////////////////////////////////////
  public void nullify() {
    graphicsArea.nullify();
  }

  /////////////////////////////////////////////////////////////////////////////
  /** This nullifies a specific data set in a plot by setting its arrays 
  *   contained in graphicsArea.x and graphicsArea.y to null */
  /////////////////////////////////////////////////////////////////////////////
  public void nullify(int i) {
    graphicsArea.nullify(i);
  }
}
