/*--------------------------------------------------------------------------
*    This file is part of InversionKit.
*
*    InversionKit is free software: you can redistribute it and/or modify
*    it under the terms of the GNU General Public License as published by
*    the Free Software Foundation, either version 3 of the License, or
*    (at your option) any later version.
*
*    InversionKit is distributed in the hope that it will be useful,
*    but WITHOUT ANY WARRANTY; without even the implied warranty of
*    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*    GNU General Public License for more details.
*
*    You should have received a copy of the GNU General Public License
*    along with InversionKit.  If not, see <http://www.gnu.org/licenses/>.
*
****************************************************************************
*
*    Copyright (c) Daniel Reese, Sergei Zharkov, 2008
*
****************************************************************************/
import inversion_applet.DataCenter;

import inversion_applet.FrequencyShiftsPage;
import inversion_applet.RotationalSplittingsPage;
import inversion_applet.RotationInversionPage;
import inversion_applet.StructuralInversionPage;
import inversion_applet.KernelPage;
import inversion_applet.ModelPage;
import javax.swing.JTabbedPane;
import javax.swing.JFrame;
import javax.swing.JApplet;
import java.awt.BorderLayout;
import java.awt.Container;

/*************************************************************************
**
**    Applet inversion
**                                              Version 1.0   Mai 2008
**
*************************************************************************/

public class InversionKit extends JApplet {

  DataCenter dataCenter;        // this contains all of the data
  ModelPage modelPage;          // this contains information on the model that is loaded
  FrequencyShiftsPage frequencyShiftsPage;  
                                // this contains information on the frequency shifts used in the inversion 
  RotationalSplittingsPage rotationalSplittingsPage;  
                                // this contains information on the rotational splittings used in the inversion 
  KernelPage kernelPage;        // this contains information on the kernels that are loaded
  RotationInversionPage rotationInversionPage;  
                                // this contains the tab with the rotation inversion results
  StructuralInversionPage structuralInversionPage;  
                                // this contains the tab with the rotation inversion results
  JTabbedPane tabbedPane;
 
  /*
  **  This is the entry point if the program is run as an applet.
  */
  public void init() {
    dataCenter = new DataCenter();
    dataCenter.isApplet = true;
    initProgram();
  }

  /*
  **  This is the entry point if the program is run as a standalone application.
  */
  public static void main(String args[]) {
    InversionKit myApplet = new InversionKit();
    myApplet.dataCenter = new DataCenter();
    myApplet.dataCenter.isApplet = false;
    myApplet.initProgram();

    JFrame f = new JFrame("InversionKit 1.0");
    f.add("Center", myApplet);
    f.setSize(800,800);
    f.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
    f.setVisible(true);
  }

  /*
  **  Initialise the program
  */
  public void initProgram() {
    rotationInversionPage = new RotationInversionPage(dataCenter);
    rotationInversionPage.initProgram();

    structuralInversionPage = new StructuralInversionPage(dataCenter);
    structuralInversionPage.initProgram();

    modelPage = new ModelPage(dataCenter);
    modelPage.initProgram();

    frequencyShiftsPage = new FrequencyShiftsPage(dataCenter);
    frequencyShiftsPage.init();

    rotationalSplittingsPage = new RotationalSplittingsPage(dataCenter);
    rotationalSplittingsPage.init();

    kernelPage = new KernelPage(dataCenter);
    kernelPage.init();

    tabbedPane = new JTabbedPane();
    tabbedPane.addTab("Rotational Inversion",rotationInversionPage);
    tabbedPane.addTab("Structural Inversion",structuralInversionPage);
    tabbedPane.addTab("Stellar model",modelPage);
    tabbedPane.addTab("Kernels", kernelPage);
    tabbedPane.addTab("Rotational splittings", rotationalSplittingsPage);
    tabbedPane.addTab("Frequency shifts", frequencyShiftsPage);

    Container c = getContentPane();
    c.setLayout(new BorderLayout());
    c.add("Center", tabbedPane);
  }  
}
