/*--------------------------------------------------------------------------
*    This file is part of InversionKit.
*
*    InversionKit is free software: you can redistribute it and/or modify
*    it under the terms of the GNU General Public License as published by
*    the Free Software Foundation, either version 3 of the License, or
*    (at your option) any later version.
*
*    InversionKit is distributed in the hope that it will be useful,
*    but WITHOUT ANY WARRANTY; without even the implied warranty of
*    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*    GNU General Public License for more details.
*
*    You should have received a copy of the GNU General Public License
*    along with InversionKit.  If not, see <http://www.gnu.org/licenses/>.
*
****************************************************************************
*
*    Copyright (c) 2008, Daniel Reese, Sergei Zharkov
*
****************************************************************************/
package inversion_applet;

import graph.IPlot;
import inversion_applet.DataCenter;
import inversion_applet.Utilities;
import java.awt.BorderLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.GridLayout;
import java.io.BufferedReader;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.IOException;
import java.io.File;
import java.io.OutputStream;
import java.io.FileOutputStream;
import java.net.URL;
import java.util.Enumeration;
import java.util.StringTokenizer;
import java.util.zip.GZIPInputStream;
import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JComponent;
import javax.swing.JDialog;
import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextField;
import javax.swing.JTree;
import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.tree.MutableTreeNode;
import javax.swing.tree.TreePath;
import javax.swing.tree.TreeSelectionModel;
import javax.swing.event.TreeSelectionEvent;
import javax.swing.event.TreeSelectionListener;

public class ModelPage extends JPanel {
  DataCenter dataCenter;

  JLabel sTitle = new JLabel();
  JLabel sMass = new JLabel();
  JLabel sRadius = new JLabel();
  JPanel buttons;
  JPanel graphArea;
  JButton LoadModel;
  JButton LoadInternetModel;
  JButton LoadHELASmodel;
  JButton DownloadHELASmodelButton;
  JButton LoadSaveHELASmodelButton;
  JButton HELAScancelButton;
  JOptionPane HELASdialog;
  JButton PolytropeButton;
  JButton ClearModel;
  JButton PrintModel;
  JTree HELAStree;
  JTextField HELASTextField;
  
  IPlot DensityPlot;
  IPlot PressurePlot;

  double[] grid, pressure, rho, Gamma1;

  /////////////////////////////////////////////////////////////////////////////
  /** A constructor */
  /////////////////////////////////////////////////////////////////////////////
  public ModelPage(DataCenter dataCenter) {
    this.dataCenter = dataCenter;
  }

  /////////////////////////////////////////////////////////////////////////////
  /** This is to test this out as a stand alone application */
  /////////////////////////////////////////////////////////////////////////////
  public static void main(String[] argv) throws IOException {
    JFrame f = new JFrame("Model reader");
    DataCenter dataCenter = new DataCenter();
    ModelPage aModel = new ModelPage(dataCenter);
    aModel.initProgram();

    // add an exit button    
    JButton QuitButton = Utilities.createButton("Exit");
    aModel.buttons.add(QuitButton);
    QuitButton.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) { System.exit(0); }
    });

    f.add("Center",aModel);
    f.setSize(220,200);
    f.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
    f.setVisible(true);
  }
  
  /////////////////////////////////////////////////////////////////////////////
  /** This is what sets up the buttons and the layout */
  /////////////////////////////////////////////////////////////////////////////
  public void initProgram() {
    initLoadModel();
    initLoadInternetModel();
    initLoadHELASmodel();
    initClearModel();
    initPolytropeButton();
    updateSLabels();
    DensityPlot = new IPlot("Density", "Radius, r/R", "Density (in g/cm\u00B3)", 1);
    DensityPlot.setIOenabled(!dataCenter.isApplet);
    PressurePlot = new IPlot("Pressure", "Radius, r/R", "Pressure (in dyn/cm\u00B2)", 1);
    PressurePlot.setIOenabled(!dataCenter.isApplet);
    
    graphArea = new JPanel();
    graphArea.setLayout(new GridLayout(2,1));
    graphArea.add(DensityPlot);
    graphArea.add(PressurePlot);

    setLayout(new BorderLayout());
    buttons = new JPanel();
    buttons.setLayout(new BoxLayout(buttons, BoxLayout.Y_AXIS));
    buttons.add(LoadModel);
    buttons.add(LoadInternetModel);
    buttons.add(LoadHELASmodel);
    buttons.add(PolytropeButton);
    buttons.add(ClearModel);
    buttons.add(new JLabel(" "));
    buttons.add(sTitle);
    buttons.add(sMass);
    buttons.add(sRadius);
    //buttons.add(PrintModel);
    
    add("Center",graphArea);
    add("East",buttons);
  }

  ////////////////////////////////////////////////////////////////////////////////////
  // This updates the information given in the sTitle, sMass, and sRadius.
  ////////////////////////////////////////////////////////////////////////////////////
  private void updateSLabels() {
    if (dataCenter.modelLoaded) {
      sTitle.setText(" The stellar parameters are:");
      sMass.setText(String.format("   Mass:   %.3f M\u2299",
                      dataCenter.StellarMass/DataCenter.SolarMass));
      sRadius.setText(String.format("   Radius: %.3f R\u2299",
                      dataCenter.StellarRadius/DataCenter.SolarRadius));
    } else {
      sTitle.setText(" No models loaded");
      sMass.setText(" ");
      sRadius.setText(" ");
    }
  }
      
  ////////////////////////////////////////////////////////////////////////////////////
  // this initialises PolytropeButton:
  ////////////////////////////////////////////////////////////////////////////////////
  private void initPolytropeButton() {
    PolytropeButton = Utilities.createButton("Generate Polytrope");
    PolytropeButton.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent ev) {
        Double N, Mass, Radius;
        Integer nstep;
        Object[] message = new Object[8];
        message[0] = "Enter the polytropic index N (0 \u2264 N \u2264 4.9):";
        message[1] = new JTextField("3.0");
        message[2] = "Enter the number of grid points n (10 \u2264 n \u2264 10000):";
        message[3] = new JTextField("1001");
        message[4] = "Enter the stellar mass M/M\u2299 (0.01 \u2264 M/M\u2299 \u2264 1000):";
        message[5] = new JTextField("1.0");
        message[6] = "Enter the stellar radius R/R\u2299 (0.01 \u2264 R/R\u2299 \u2264 1000): ";
        message[7] = new JTextField("1.0");

        String[] options = new String[2];
        options[0] = "Generate polytrope";
        options[1] = "Cancel";

        int result = JOptionPane.showOptionDialog(
          PolytropeButton,              // the parent that the dialog blocks
          message,                      // the dialog message array
          "Generate a polytrope",       // the title of the dialog window
          JOptionPane.DEFAULT_OPTION,   // option type
          JOptionPane.QUESTION_MESSAGE, // message type
          null,                         // optional icon, use null to use the default icon
          options,                      // options string array, will be made into buttons
          options[0]                    // option that should be made into a default button
        );
        
        switch(result) {
         case 1:  // Cancel
           break;

         default: // Generate polytrope

           // Check to see if different entries are acceptable:

           N     = Utilities.readDouble(((JTextField)message[1]).getText(),
                   "polytropic index",0.0,4.9,PolytropeButton);
           if (N == null) return;
           nstep = Utilities.readInt(((JTextField)message[3]).getText(),
                   "number of grid points",10,10000,PolytropeButton);
           if (nstep == null) return;
           Mass  = Utilities.readDouble(((JTextField)message[5]).getText(),
                   "stellar mass",0.01,1000,PolytropeButton);
           if (Mass == null) return;
           Radius= Utilities.readDouble(((JTextField)message[7]).getText(),
                   "stellar radius",0.01,1000,PolytropeButton);
           if (Radius == null) return;
           
           // set the stellar mass:
           dataCenter.StellarMass  = Mass*DataCenter.SolarMass;

           // set the stellar radius:
           dataCenter.StellarRadius= Radius*DataCenter.SolarRadius;

           // generate the polytrope:
           generatePolytrope(nstep,N);
           dataCenter.modelLoaded = true;
           dataCenter.rotationInversionUpToDate = false;
           dataCenter.structInversionUpToDate = false;
           refreshPlots();
           updateSLabels();
           break;
        }
      }
    });
  }

  /////////////////////////////////////////////////////////////////////////////
  // this initialises the LoadModel button
  /////////////////////////////////////////////////////////////////////////////
  private void initLoadModel() {
    LoadModel = Utilities.createButton("Load model");
    LoadModel.setEnabled(!dataCenter.isApplet);
    LoadModel.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {

        // open in current directory:
        JFileChooser fc = new JFileChooser(dataCenter.currentDirectory);

        // show the filechooser
        int result = fc.showOpenDialog(LoadModel);  // put parent component
        // get filename
        if(result == JFileChooser.APPROVE_OPTION) {
          File aFile = fc.getSelectedFile();

          // check to see if file exists:
          if (aFile.exists()) {
            readModel(aFile.getPath(),false);
            dataCenter.currentDirectory = aFile.getParentFile();
          } else {
            JOptionPane.showMessageDialog(
              LoadModel,
              "File \""+aFile.getName()+"\" doesn't exist.",
              "File not found",
              JOptionPane.ERROR_MESSAGE
            );
          }
        }
      }
    });
  }
    
  /////////////////////////////////////////////////////////////////////////////
  // this initialises the LoadModel button
  /////////////////////////////////////////////////////////////////////////////
  private void initLoadInternetModel() {
    LoadInternetModel = Utilities.createButton("Load model from internet");
    LoadInternetModel.setEnabled(!dataCenter.isApplet);
    LoadInternetModel.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        String urlAddress = JOptionPane.showInputDialog(
          LoadInternetModel,
          "Please enter the URL of the model\n"+
          "you would like to download:",
          "Download model",
          JOptionPane.QUESTION_MESSAGE
        );
        readModel(urlAddress,true);
      }
    });
  }

  /////////////////////////////////////////////////////////////////////////////
  // this initialises the LoadModel button
  /////////////////////////////////////////////////////////////////////////////
  private void initLoadHELASmodel() {
    // Initialise some buttons and a text field:
    LoadHELASmodel = Utilities.createButton("Load HELAS model");
    LoadHELASmodel.setEnabled(!dataCenter.isApplet);
    DownloadHELASmodelButton = new JButton("Load model");
    DownloadHELASmodelButton.setEnabled(false);
    LoadSaveHELASmodelButton = new JButton("Load and save model");
    LoadSaveHELASmodelButton.setEnabled(false);
    HELAScancelButton = new JButton("Cancel");
    HELASTextField = new JTextField();
    HELASTextField.setEditable(false);

    // initialise tree with list of models and assocated scroll pane:
    initHELAStree();
    JScrollPane aScroller = new JScrollPane();
    aScroller.getViewport().add(HELAStree);

    // initialise dialog which allows the user to select and download
    // a helas model:
    Object[] message = new Object[5];
    message[0] = "Please select one of the following models:";
    message[1] = "(hint: using the arrow keys is an\n"+
                 "efficient way to navigate the tree)";
    message[2] = aScroller;
    message[3] = "Selected model:";
    message[4] = HELASTextField;

    Object[] options = new Object[3];
    options[0] = DownloadHELASmodelButton;
    options[1] = LoadSaveHELASmodelButton;
    options[2] = HELAScancelButton;
    HELASdialog = new JOptionPane(
      message,                      // the dialog message array
      JOptionPane.QUESTION_MESSAGE, // message type
      JOptionPane.DEFAULT_OPTION,   // option type
      null,                         // optional icon, use null to use the default icon
      options,                      // options string array, will be made into buttons
      options[0]                    // default option
    );
    
    // add some action listeners to different buttons:    
    DownloadHELASmodelButton.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        readModel(HELASTextField.getText(),true);
        HELASdialog.setValue(0);
      }
    });

    // add some action listeners to different buttons:    
    LoadSaveHELASmodelButton.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        // open in current directory:
        JFileChooser fc = new JFileChooser(dataCenter.currentDirectory);

        // show the filechooser
        int result = fc.showSaveDialog(LoadSaveHELASmodelButton);  // put parent component

        // get filename
        if(result == JFileChooser.APPROVE_OPTION) {
          File aFile = fc.getSelectedFile();

          // check to see if file exists:
          if (aFile.exists()) {
            result = JOptionPane.showConfirmDialog(
                 LoadSaveHELASmodelButton,
                 "Are you sure you want to overwrite "+aFile.getName()+"?",
                 "File exists",JOptionPane.YES_NO_OPTION);
          } else {
            result = JOptionPane.YES_OPTION;
          }

          if(result == JOptionPane.YES_OPTION)
            downloadModel(HELASTextField.getText(),aFile.getPath());

          if (aFile.exists()) {
            dataCenter.currentDirectory = aFile.getParentFile();
            readModel(aFile.getPath(),false);
            HELASdialog.setValue(1);
          }
        }
      }
    });

    HELAScancelButton.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        HELASdialog.setValue(2);
      }
    });

    LoadHELASmodel.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {

        JDialog dialog = HELASdialog.createDialog(LoadHELASmodel, "Download a HELAS model");
        dialog.setSize(500,500);
        dialog.setResizable(true);
        dialog.setVisible(true);
      }
    });
  }

  /////////////////////////////////////////////////////////////////////////////
  // This downloads a file from the HELAS platform and saves it on the hard
  // disk.  If the filename doesn't end with ".gz", it is decompressed
  // beforehand.
  /////////////////////////////////////////////////////////////////////////////
  private void downloadModel(String url, String filename) {
    try {
      if (filename.endsWith(".gz")) {
        // just download
        InputStream in = new URL(url).openStream();
        OutputStream out = new FileOutputStream(filename);
        // transfer bytes from in to out:
        byte[] buf = new byte[1024];
        int len;
        while ((len = in.read(buf)) > 0) out.write(buf, 0, len);
        in.close();
        out.close();
      } else {
        // download and decompress
        InputStream aux1 = new URL(url).openStream();
        GZIPInputStream in = new GZIPInputStream(aux1);
        OutputStream out = new FileOutputStream(filename);
        // transfer bytes from in to out:
        byte[] buf = new byte[1024];
        int len;
        while ((len = in.read(buf)) > 0) out.write(buf, 0, len);
        in.close();
        out.close();
      }
    } catch(Exception e) {
      JOptionPane.showMessageDialog(
               LoadSaveHELASmodelButton,  // the parent component
               "Unable to download and/or save HELAS model.",
               "Error",
               JOptionPane.ERROR_MESSAGE
             );
    }
  }

  /////////////////////////////////////////////////////////////////////////////
  // this initialises the LoadModel button
  /////////////////////////////////////////////////////////////////////////////
  private void initClearModel() {
    ClearModel = Utilities.createButton("Clear model");
    ClearModel.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        dataCenter.modelLoaded = false;
        dataCenter.rotationInversionUpToDate = false;
        dataCenter.structInversionUpToDate = false;

        dataCenter.StellarRadius = 0.0;
        dataCenter.StellarMass = 0.0;
        dataCenter.Pref = 0.0;
        dataCenter.RhoRef = 0.0;

        dataCenter.gridModel = null;
        dataCenter.Gamma1 = null;
        dataCenter.rho = null;
        dataCenter.pressure = null;
        dataCenter.c2 = null;
        
        DensityPlot.nullify();
        DensityPlot.repaint();
        PressurePlot.nullify();
        PressurePlot.repaint();

        updateSLabels();
      }
    });
  }
   
  /**********************************************************
  * This subroutine takes in a a file "aFile" and determines
  * which format it's in and whether or not it should be
  * decompressed.  It then call up the appropriate method
  * to read it (depending on the format).
  **********************************************************/ 
  private void readModel(String fileOrURL, boolean fromInternet) {
    BufferedReader modelInput;
  
    try {

      if (fromInternet) {
        modelInput = Utilities.OpenURL(fileOrURL);
      } else {
        modelInput = Utilities.OpenFile(fileOrURL);
      }

      // This next part determines the format of the model by
      // looking for the subtring "CESAM" on the second line.

      // The argument is the limit on the number of characters
      // that may be read while still preserving the mark.
      // Putting a large number may require increasing the size
      // of the buffer.
      modelInput.mark(500);

      String line;
      line = modelInput.readLine();
      line = modelInput.readLine();
      modelInput.reset();
      if (line.indexOf("CESAM")>=0) {
        readOSCmodel(modelInput);
      } else {
        readFAMDLmodel(modelInput);
      }

    } catch(Exception e) {
      JOptionPane.showMessageDialog(
        LoadModel,  // the parent component
        "Unable to read from:\n  "+fileOrURL+"\n",
        "Error opening file",
        JOptionPane.ERROR_MESSAGE
      );
    }
  }    

  /**********************************************************
  * This subroutine reads an FAMDL model from ASTEC
  * and stores the information in different arrays.
  **********************************************************/ 
  public void readFAMDLmodel(BufferedReader modelInput) {

    int[] ndimin = new int[4];
    double[] xdatain = new double[9];
    double[][] xin;
    String line;

    try{
      line = modelInput.readLine();
      StringTokenizer t = new StringTokenizer(line," ");
      for(int i=1; i<4; i++) {ndimin[i] = Integer.parseInt(t.nextToken());}

      line = modelInput.readLine();
      xdatain[1] = Double.parseDouble(line.substring( 0,20));
      xdatain[2] = Double.parseDouble(line.substring(20,40));
      xdatain[3] = Double.parseDouble(line.substring(40,60));
      xdatain[4] = Double.parseDouble(line.substring(60,80));

      line = modelInput.readLine();
      xdatain[5] = Double.parseDouble(line.substring( 0,20));
      xdatain[6] = Double.parseDouble(line.substring(20,40));
      xdatain[7] = Double.parseDouble(line.substring(40,60));
      xdatain[8] = Double.parseDouble(line.substring(60,80));

      xin = new double[ndimin[3]+1][ndimin[2]];
      int k = 0;
      for(int i=0; i<ndimin[2]; i++) {
        for(int j=0; j<(ndimin[3]+1); j++) {
          if (k==0) line = modelInput.readLine();
          xin[j][i] = Double.parseDouble(line.substring(k*20,(k+1)*20));
          k++;
          if (k==4) k=0;
        }
      }
      modelInput.close();
    } catch (Exception e) {
      JOptionPane.showMessageDialog(
        LoadModel,  // the parent component
        "File format not recognised",
        "Error reading model",
        JOptionPane.ERROR_MESSAGE
      );
      return;
    }

    // Stellar mass in g:
    dataCenter.StellarMass = xdatain[1];
    
    // Stellar radius in cm:
    dataCenter.StellarRadius = xdatain[2];

    // Reference pressure:
    dataCenter.Pref = DataCenter.G*Math.pow(dataCenter.StellarMass,2.0)
                                  /Math.pow(dataCenter.StellarRadius,4.0);

    // Reference density:
    dataCenter.RhoRef = dataCenter.StellarMass
                       /Math.pow(dataCenter.StellarRadius,3.0);

    // the grid (it's already non-dimensional):
    dataCenter.gridModel = new double[ndimin[2]];
    for(int i=0; i<ndimin[2]; i++) dataCenter.gridModel[i] = xin[0][i];
    
    // the normalised pressure:
    dataCenter.pressure = new double[ndimin[2]];
    for(int i=1; i<ndimin[2]; i++) 
      dataCenter.pressure[i] = xin[1][i]*xin[1][i]*xin[5][i]*xin[0][i]*xin[0][i]
                             / (4.0*Math.PI*xin[2][i]*xin[3][i]);
    // special treatment for the center:
    dataCenter.pressure[0] = xdatain[3]/dataCenter.Pref;

    // the normalised density:
    dataCenter.rho = new double[ndimin[2]];
    for(int i=0; i<ndimin[2]; i++)
      dataCenter.rho[i] = xin[1][i]*xin[5][i]/(4.0*Math.PI);

    // the adiabatic exponent:
    dataCenter.Gamma1 = new double[ndimin[2]];
    for(int i=0; i<ndimin[2]; i++)
      dataCenter.Gamma1[i] = xin[3][i];

    // the normalised sound velocity:
    dataCenter.c2 = new double[ndimin[2]];
    for(int i=1; i<ndimin[2]; i++)
      dataCenter.c2[i] = xin[0][i]*xin[0][i]*xin[1][i]/xin[2][i];
    // special treatment for the center:
    dataCenter.c2[0] = xin[3][0]*dataCenter.pressure[0]/dataCenter.rho[0];

    dataCenter.rotationInversionUpToDate = false;
    dataCenter.structInversionUpToDate = false;
    dataCenter.modelLoaded = true;
    updateSLabels();
    refreshPlots();
  }    
  
  /**********************************************************
  * This subroutine reads a OSC model from CESAM
  * and stores the information in different arrays.
  **********************************************************/ 
  public void readOSCmodel(BufferedReader modelInput) {
    
    int[] ndimin = new int[5];
    double[] xdatain;  // the dimensions are set when the model is read
    double[][] xin;    // the dimensions are set when the model is read
    String line;

    try{
      // Skip the header
      for(int i=0; i<5; i++) {line = modelInput.readLine();}

      // Now read lines of text:
      line = modelInput.readLine();
      StringTokenizer t = new StringTokenizer(line," ");
      for(int i=0; i<5; i++) {ndimin[i] = Integer.parseInt(t.nextToken());}

      // I couldn't find any readf methods, so I cut the string up into
      // substrings of 19 characters and then try to read a value.
      //
      // Note: the StringTokenizer method won't work because there
      //       are no white spaces preceding negative values (since
      //       the minus sign occupies this space).

      int jj;
      // I add an extra element to xdatain so as to agree with CESAM
      // documentation.  xdatain[0] corresponds to nothing.
      xdatain = new double[ndimin[1]+1];
      for(int i=0; i<ndimin[1];i++) {
        jj=i%5;
        if(jj==0) line = modelInput.readLine();
        xdatain[i+1] = Double.parseDouble(line.substring(jj*19,(jj+1)*19));
      }

      // I increase the first dimension of xin by 1, so as to
      // have the same first index as what is given in the Cesam
      // documentation.  Therefore, xin[0][*] corresponds to nothing:
      xin = new double[ndimin[2]+ndimin[3]+1][ndimin[0]];

      // Cesam Models start from the surface and go to the center,
      // hence the reverse direction on the i loop.
      for(int i=ndimin[0]-1; i>=0;i--) {
        for(int j=0; j<(ndimin[2]+ndimin[3]); j++) {
          jj=j%5;
          if(jj==0) line = modelInput.readLine();
          // The "j+1" coordinate insures that the first index of
          // of xin corresponds to Cesam documentation.
          xin[j+1][i] = Double.parseDouble(line.substring(jj*19,(jj+1)*19));
        }
      }
    } catch(Exception e) {
      JOptionPane.showMessageDialog(
        LoadModel,  // the parent component
        "File format not recognised",
        "Error reading model",
        JOptionPane.WARNING_MESSAGE
      );
      return;
    }

    // Stellar mass in g:
    dataCenter.StellarMass = xdatain[1];

    // Stellar radius in cm (does not correspond exactly to last grid point):
    dataCenter.StellarRadius = xdatain[2];
    
    // Radial grid, normalised by stellar radius:
    dataCenter.gridModel = new double[ndimin[0]];
    for (int i=0; i < ndimin[0]; i++) 
      dataCenter.gridModel[i] = xin[1][i]/dataCenter.StellarRadius;

    // Normalised pressure profile and reference value for the pressure
    dataCenter.Pref = DataCenter.G*Math.pow(dataCenter.StellarMass,2.0)
                                  /Math.pow(dataCenter.StellarRadius,4.0);
    dataCenter.pressure = new double[ndimin[0]];
    for (int i=0; i < ndimin[0]; i++)
      dataCenter.pressure[i] = xin[4][i]/dataCenter.Pref;

    // Normalised density profile and reference value for the density:
    dataCenter.rho = new double[ndimin[0]];
    dataCenter.RhoRef = dataCenter.StellarMass
                       /Math.pow(dataCenter.StellarRadius,3.0);
    for (int i=0; i < ndimin[0]; i++) 
      dataCenter.rho[i] = xin[5][i]/dataCenter.RhoRef;

    // Gamma1 profile:                                    
    dataCenter.Gamma1 = new double[ndimin[0]];
    for (int i=0; i < ndimin[0]; i++) dataCenter.Gamma1[i] = xin[10][i];
    
    // Normalised squared sound velocity profile:
    dataCenter.c2 = new double[ndimin[0]];
    for (int i=0; i < ndimin[0]; i++)
      dataCenter.c2[i] = dataCenter.Gamma1[i]*dataCenter.pressure[i]
                        /dataCenter.rho[i];

    dataCenter.rotationInversionUpToDate = false;
    dataCenter.structInversionUpToDate = false;
    dataCenter.modelLoaded = true;
    updateSLabels();
    refreshPlots();
  }

  /**********************************************************
  * This generates a polytropic model (the actual
  * calculations are done in the Polytrope class):
  **********************************************************/ 
  private void generatePolytrope(int nstep, double N) {

    // this generates the polytrope:
    Polytrope p = new Polytrope(nstep, N);
    N = p.getN(); // just in case N is modified in the previous step
    
    // put the results in dataCenter:

    // reference value for the pressure:    
    dataCenter.Pref = DataCenter.G*Math.pow(dataCenter.StellarMass,2.0)
                                  /Math.pow(dataCenter.StellarRadius,4.0);

    // reference value for the density:
    dataCenter.RhoRef = dataCenter.StellarMass
                       /Math.pow(dataCenter.StellarRadius,3.0);

    // the radial grid:
    dataCenter.gridModel = new double[nstep];
    for(int i=0; i<nstep; i++) dataCenter.gridModel[i] = p.r[i];
    
    // the adiabatic exponent (= 5/3 throughout):
    double Gamma1 = 5.0/3.0;
    dataCenter.Gamma1 = new double[nstep];
    for(int i=0; i<nstep; i++) dataCenter.Gamma1[i] = Gamma1;
    
    // the normalised density profile:
    double RhoC = 3.0*p.Alpha/(4.0*Math.PI);
    dataCenter.rho = new double[nstep];
    for(int i=0; i<nstep; i++) dataCenter.rho[i] = RhoC*Math.pow(Math.abs(p.h[i]),N);

    // the normalised pressure profile:
    double PC = 9.0*p.Alpha*p.Alpha/(4.0*Math.PI*(N+1.0)*p.Xi*p.Xi);
    dataCenter.pressure = new double[nstep];
    for(int i=0; i<nstep; i++) dataCenter.pressure[i] = PC*Math.pow(Math.abs(p.h[i]),N+1);

    // the normalised sound velocity profile:
    double c2C = 3.0*Gamma1*p.Alpha/((N+1.0)*p.Xi*p.Xi);
    dataCenter.c2 = new double[nstep];
    for(int i=0; i<nstep; i++) dataCenter.c2[i] = c2C*Math.abs(p.h[i]);
  }

  /**********************************************************
  * This creates initialises HELAS tree with the stellar
  * models from the HELAS website.
  **********************************************************/ 
  private void initHELAStree() {
    HELAStree = new JTree(readHELASlist());
    HELAStree.getSelectionModel().setSelectionMode(TreeSelectionModel.SINGLE_TREE_SELECTION);
    HELAStree.addTreeSelectionListener(new TreeSelectionListener() {
      public void valueChanged(TreeSelectionEvent e) {
        TreePath aPath = HELAStree.getSelectionPath();
        HELASTextField.setText(Path2URL(aPath));
        DownloadHELASmodelButton.setEnabled(((MutableTreeNode)aPath.getLastPathComponent()).isLeaf());
        LoadSaveHELASmodelButton.setEnabled(((MutableTreeNode)aPath.getLastPathComponent()).isLeaf());
      }
    });
  }

  /**********************************************************
  * This reads the list of HELAS models from a file called
  * model_list in the ressource_files directory.
  **********************************************************/ 
  private DefaultMutableTreeNode readHELASlist() {
    DefaultMutableTreeNode root = new DefaultMutableTreeNode("HELAS stellar model");
    DefaultMutableTreeNode previousNode;
    DefaultMutableTreeNode currentNode;
    DefaultMutableTreeNode nextNode;
    String anElement;
    Enumeration en;
    String line;
    BufferedReader list;

    try {
      // Open file with the list of HELAS models
      URL url = getClass().getResource("/resource_files/model_list");
      InputStream is = url.openStream();
      InputStreamReader isr = new InputStreamReader(is);
      list = new BufferedReader(isr);
    } catch (Exception e) {
      return root;
    }
    try {
      while (true) {
        line = list.readLine();  // read a line of text
        if (line == null) break; // test for the end of the buffer
        StringTokenizer t =  new StringTokenizer(line,"/");
        previousNode = root;

        while(t.hasMoreTokens()) {
          anElement = t.nextToken();

          // test to see if previousNode already has current node:
          nextNode = null;
          for (en = previousNode.children();en.hasMoreElements() ;) {
            currentNode = (DefaultMutableTreeNode)en.nextElement();
            if (currentNode.toString().equals(anElement)) {
              nextNode = currentNode;
              break;
            }
          }

          // node if it doesn't exist yet:
          if (nextNode == null) {
            nextNode = new DefaultMutableTreeNode(anElement);
            previousNode.add(nextNode);
          }

          previousNode = nextNode;
        }
      }
      list.close();
    } catch (Exception e) {
      return root;
    }
    return root;
  }

  /**********************************************************
  * This converts a tree path from the HELAS tree into the
  * correct URL address.
  **********************************************************/ 
  private static String Path2URL(TreePath aPath) {
    String result = "http://www.astro.up.pt/helas/stars/cesam/";
    if (aPath == null) return result;
    Object[] pathArray = aPath.getPath();
    if (pathArray.length < 2) return result;
    result = result + pathArray[1].toString() + "/data";
    for(int i=2; i<pathArray.length; i++)
      result = result + "/"+pathArray[i].toString();
    return result;
  }

  /**********************************************************
  * This refreshes the density and pressure plots and writes
  * some information in the output log.
  **********************************************************/ 
  private void refreshPlots() {

    if (dataCenter.modelLoaded) {
      double[] aux = new double[dataCenter.gridModel.length];
      for(int i=0; i<aux.length; i++) aux[i] = dataCenter.RhoRef*dataCenter.rho[i];
      DensityPlot.replaceDataSet(0,dataCenter.gridModel,aux);
      DensityPlot.findRange();
      DensityPlot.repaint();

      for(int i=0; i<aux.length; i++) aux[i] = dataCenter.Pref*dataCenter.pressure[i];
      PressurePlot.replaceDataSet(0,dataCenter.gridModel,aux);
      PressurePlot.findRange();
      PressurePlot.repaint();
    }
  }
}
