/*--------------------------------------------------------------------------
*    This file is part of InversionKit.
*
*    InversionKit is free software: you can redistribute it and/or modify
*    it under the terms of the GNU General Public License as published by
*    the Free Software Foundation, either version 3 of the License, or
*    (at your option) any later version.
*
*    InversionKit is distributed in the hope that it will be useful,
*    but WITHOUT ANY WARRANTY; without even the implied warranty of
*    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*    GNU General Public License for more details.
*
*    You should have received a copy of the GNU General Public License
*    along with InversionKit.  If not, see <http://www.gnu.org/licenses/>.
*
****************************************************************************
*
*    Copyright (c) Daniel Reese, Sergei Zharkov, 2008
*
****************************************************************************/
package inversion_applet;

import inversion_applet.DataCenter;
import inversion_applet.FrequencyShiftsDataModel;
import inversion_applet.Utilities;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import java.awt.BorderLayout;
import javax.swing.JOptionPane;
import javax.swing.JFileChooser;
import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.BoxLayout;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.JPanel;
import javax.swing.JComponent;
import java.io.File;

public class RotationalSplittingsPage extends JPanel {
  private JLabel numberRS = new JLabel();
  private JTable  rotationalSplittingsTable;
  private JScrollPane tableScroller;
  private JPanel buttonBox = new JPanel();
  private JButton addRowButton;
  private JButton clearButton;
  private JButton sortButton;
  private JButton loadFileButton;
  private JButton appendFileButton;
  private JButton writeFileButton;
  private JButton removeRowsButton;
  private JButton calculateRSbutton;
  private RotationalSplittingsDataModel rotationalSplittingsData;
  private DataCenter dataCenter;

  public RotationalSplittingsPage(DataCenter dataCenter) {
    this.dataCenter = dataCenter;
    dataCenter.rotationalSplittingsPage = this;
  }

  private void createTable() {
    // Create the table data model:
    rotationalSplittingsData = new RotationalSplittingsDataModel(dataCenter,buttonBox);

    // Create the table
    rotationalSplittingsTable = new JTable(rotationalSplittingsData);
    
    // Create a scroll pane for the table
    tableScroller = new JScrollPane(rotationalSplittingsTable);
    return;
  }

  /** This sets up the user graphical interface */
  public void init() {
    initAddRowButton();
    initRemoveRowsButton();
    initClearButton();
    initSortButton();
    initOpenFileButtons();
    initWriteFileButton();
    initCalculateRSbutton();

    buttonBox.setLayout(new BoxLayout(buttonBox, BoxLayout.Y_AXIS));
    buttonBox.add(addRowButton);
    buttonBox.add(removeRowsButton);
    buttonBox.add(clearButton);
    buttonBox.add(sortButton);
    buttonBox.add(new JLabel(" "));
    buttonBox.add(loadFileButton);
    buttonBox.add(appendFileButton);
    buttonBox.add(writeFileButton);
    buttonBox.add(new JLabel(" "));
    buttonBox.add(calculateRSbutton);
    buttonBox.add(new JLabel(" "));
    buttonBox.add(numberRS);

    setLayout(new BorderLayout());
    createTable();
    updateNumberRS();
    add("Center",tableScroller);
    add("East",buttonBox);
  }

  // this update the text in numberRS:
  private void updateNumberRS() {
    numberRS.setText(String.format(" Number of R.S: %d",
                     rotationalSplittingsData.getRowCount()));
  }
  
  // this initialises addRowButton
  private void initAddRowButton() {
    addRowButton = Utilities.createButton("Add row");
    addRowButton.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        rotationalSplittingsData.addBlankRow(); // add a row
        rotationalSplittingsTable.updateUI();   // update the table
        updateNumberRS();
      }
    });
  }
  
  // this initialises removeRowsButton
  private void initRemoveRowsButton() {
    removeRowsButton = Utilities.createButton("Remove selected row(s)");
    removeRowsButton.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        // If a cell is being edited, the editing must first
        // stop before any rows can be removed.
        rotationalSplittingsTable.removeEditor();
 
        // find out which rows are selected
        int[] sr = rotationalSplittingsTable.getSelectedRows();

        // This clears the selection (so as to avoid having
        // non-existent rows selected):
        rotationalSplittingsTable.clearSelection();

        // if the entire table is selected, just clear
        // the data (this is more rapid and sets the
        // correct value for dataCenter.RSloaded):
        if (sr.length == dataCenter.nRS.size()) {
          rotationalSplittingsData.clear();
          return;
        }
            
        // Remove the selected rows (the loop is backwards
        // to avoid problems with modified indices):
        for(int i = sr.length-1; i >= 0; i--) 
          rotationalSplittingsData.removeRow(sr[i]);

        // this updates the table:
        rotationalSplittingsTable.updateUI();

        updateNumberRS();
      }
    });
  }

  // this initialises clearButton
  private void initClearButton() {
    clearButton = Utilities.createButton("Clear");
    clearButton.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        rotationalSplittingsTable.removeEditor();
        rotationalSplittingsTable.clearSelection();
        rotationalSplittingsData.clear();     // clear the data
        rotationalSplittingsTable.updateUI(); // update the table
        updateNumberRS();
      }
    });
  }

  // this initialises clearButton
  private void initSortButton() {
    sortButton = Utilities.createButton("Sort");
    sortButton.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        rotationalSplittingsData.sort();       // sort the data
        rotationalSplittingsTable.updateUI();  // update the table
      }
    });
  }

  // this initialises calculateRSbutton
  private void initCalculateRSbutton() {
    calculateRSbutton = Utilities.createButton("Find R.S. from target \u03A9 function");
    calculateRSbutton.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        calculateRS(calculateRSbutton);
      }
    });
  }

  // this calculates the rotational splittings from a target function
  protected void calculateRS(JComponent c) {
    rotationalSplittingsData.calculateRSfromRotaTarget(c);
    rotationalSplittingsTable.updateUI();
    updateNumberRS();
  }

  // this initialises loadFileButton
  private void initOpenFileButtons() {
    appendFileButton = Utilities.createButton("Append file");
    appendFileButton.setEnabled(!dataCenter.isApplet);
    loadFileButton  = Utilities.createButton("Load file");
    loadFileButton.setEnabled(!dataCenter.isApplet);
    ActionListener l = new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        JButton source = (JButton) e.getSource();

        // open in current directory:
        JFileChooser fc = new JFileChooser(dataCenter.currentDirectory);

        // show the filechooser
        int result = fc.showOpenDialog(source);  // put parent component

        // get filename
        if(result == JFileChooser.APPROVE_OPTION) {
          File aFile = fc.getSelectedFile();

          // check to see if file exists:
          if (aFile.exists()) {
            if (source == loadFileButton) rotationalSplittingsData.clear();
            rotationalSplittingsData.loadFile(aFile.getPath()); // load a file:
            rotationalSplittingsTable.updateUI();               // this updates the table:
            updateNumberRS();
            dataCenter.currentDirectory = aFile.getParentFile();
          } else {
            JOptionPane.showMessageDialog(
              source,
              "File \""+aFile.getName()+"\" doesn't exist.",
              "File not found",
              JOptionPane.WARNING_MESSAGE
            );
          }
        }
      }
    };
    loadFileButton.addActionListener(l);
    appendFileButton.addActionListener(l);
  }
  
  // this initialises writeFileButton
  private void initWriteFileButton() {
    writeFileButton  = Utilities.createButton("Write file");
    writeFileButton.setEnabled(!dataCenter.isApplet);
    writeFileButton.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {

        // open in current directory:
        JFileChooser fc = new JFileChooser(dataCenter.currentDirectory);

        // show the filechooser
        int result = fc.showSaveDialog(writeFileButton);  // put parent component

        // get filename
        if(result == JFileChooser.APPROVE_OPTION) {
          File aFile = fc.getSelectedFile();

          // check to see if file exists:
          if (aFile.exists()) {
            result = JOptionPane.showConfirmDialog(writeFileButton,
                 "Are you sure you want to overwrite "+aFile.getName()+"?",
                 "File exists",JOptionPane.YES_NO_OPTION);
          } else {
            result = JOptionPane.YES_OPTION;
          }

          if(result == JOptionPane.YES_OPTION)
            rotationalSplittingsData.writeFile(aFile.getPath());

          if (aFile.exists())
            dataCenter.currentDirectory = aFile.getParentFile();  
        }
      }
    });
  }
}
