/*--------------------------------------------------------------------------
*    This file is part of InversionKit.
*
*    InversionKit is free software: you can redistribute it and/or modify
*    it under the terms of the GNU General Public License as published by
*    the Free Software Foundation, either version 3 of the License, or
*    (at your option) any later version.
*
*    InversionKit is distributed in the hope that it will be useful,
*    but WITHOUT ANY WARRANTY; without even the implied warranty of
*    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*    GNU General Public License for more details.
*
*    You should have received a copy of the GNU General Public License
*    along with InversionKit.  If not, see <http://www.gnu.org/licenses/>.
*
****************************************************************************
*
*    Copyright (c) Daniel Reese, Sergei Zharkov, 2008
*
****************************************************************************/
/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////
/*
 * @(#)OptionPaneDemo.java      1.9 04/07/26
 *
 * Copyright (c) 2004 Sun Microsystems, Inc. All Rights Reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * -Redistribution of source code must retain the above copyright notice, this
 *  list of conditions and the following disclaimer.
 *
 * -Redistribution in binary form must reproduce the above copyright notice,
 *  this list of conditions and the following disclaimer in the documentation
 *  and/or other materials provided with the distribution.
 *
 * Neither the name of Sun Microsystems, Inc. or the names of contributors may
 * be used to endorse or promote products derived from this software without
 * specific prior written permission.
 *
 * This software is provided "AS IS," without a warranty of any kind. ALL
 * EXPRESS OR IMPLIED CONDITIONS, REPRESENTATIONS AND WARRANTIES, INCLUDING
 * ANY IMPLIED WARRANTY OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE
 * OR NON-INFRINGEMENT, ARE HEREBY EXCLUDED. SUN MIDROSYSTEMS, INC. ("SUN")
 * AND ITS LICENSORS SHALL NOT BE LIABLE FOR ANY DAMAGES SUFFERED BY LICENSEE
 * AS A RESULT OF USING, MODIFYING OR DISTRIBUTING THIS SOFTWARE OR ITS
 * DERIVATIVES. IN NO EVENT WILL SUN OR ITS LICENSORS BE LIABLE FOR ANY LOST
 * REVENUE, PROFIT OR DATA, OR FOR DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL,
 * INCIDENTAL OR PUNITIVE DAMAGES, HOWEVER CAUSED AND REGARDLESS OF THE THEORY
 * OF LIABILITY, ARISING OUT OF THE USE OF OR INABILITY TO USE THIS SOFTWARE,
 * EVEN IF SUN HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
 *
 * You acknowledge that this software is not designed, licensed or intended
 * for use in the design, construction, operation or maintenance of any
 * nuclear facility.
 */

/*
 * @(#)OptionPaneDemo.java      1.9 04/07/26
 */

/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////
/*
 * Copyright (c) Ian F. Darwin, http://www.darwinsys.com/, 1996-2002.
 * All rights reserved. Software written by Ian F. Darwin and others.
 * $Id: LICENSE,v 1.8 2004/02/09 03:33:38 ian Exp $
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS''
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
 * TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS
 * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 * 
 * Java, the Duke mascot, and all variants of Sun's Java "steaming coffee
 * cup" logo are trademarks of Sun Microsystems. Sun's, and James Gosling's,
 * pioneering role in inventing and promulgating (and standardizing) the Java 
 * language and environment is gratefully acknowledged.
 * 
 * The pioneering role of Dennis Ritchie and Bjarne Stroustrup, of AT&T, for
 * inventing predecessor languages C and C++ is also gratefully acknowledged.
 */

/*
 * Read some data from a gzip file.
 * 
 * @author Ian F. Darwin, http://www.darwinsys.com/
 * @version $Id: ReadGZIP.java,v 1.4 2004/03/06 20:54:38 ian Exp $
 */
/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////


package inversion_applet;

import javax.swing.JButton;
import javax.swing.JComponent;
import javax.swing.JOptionPane;
import java.awt.Dimension;

import java.io.FileReader;
import java.io.BufferedReader;
import java.io.FileInputStream;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.util.zip.GZIPInputStream;
import java.io.IOException;
import java.net.URL;

public class Utilities {

  /**********************************************************
  * This creates buttons of the right size for a BoxLayout. 
  * I don't know how it works, but simply copied it from JDK
  * SwingSet2 demo program. I modified it to include s as a
  * button title.
  **********************************************************/ 
  public static JButton createButton(String s) {
    JButton b = new JButton(s) {
      public Dimension getMaximumSize() {
        int width = Short.MAX_VALUE;
        int height = super.getMaximumSize().height;
        return new Dimension(width, height);
      }
    };
    return b;
  }

  /**********************************************************
  * This opens a file into a buffered reader.  If the file
  * is gzipped, it will first be decompressed.
  **********************************************************/ 
  public static BufferedReader OpenFile(String Filename) throws IOException {
    // determine whether the file is compressed
    if (Filename.endsWith(".gz")) {

      // the file is compressed:
      // The BufferedReader puts the file in lines.
      // The InputStreamReader does Unicode conversion, and the
      // GZipInputStream "gunzip"s the data from the FileInputStream.
      
      InputStream aux1 = new FileInputStream(Filename);
      GZIPInputStream aux2 = new GZIPInputStream(aux1);
      InputStreamReader aux3 = new InputStreamReader(aux2);
      return new BufferedReader(aux3);
    
    } else {
    
      // the file is not compressed:
      return new BufferedReader(new FileReader(Filename));
    }
  }    


  /**********************************************************
  * This opens a URL into a buffered reader.  If the URL
  * corresponds to a gzipped file, the file will first be
  * decompressed.
  **********************************************************/ 
  public static BufferedReader OpenURL(String urlAddress) throws IOException {
    // determine whether the URL contains a compressed file
    if (urlAddress.endsWith(".gz")) {

      // the file is compressed:
      // The BufferedReader puts the file in lines.
      // The InputStreamReader does Unicode conversion, and the
      // GZipInputStream "gunzip"s the data from the FileInputStream.
      
      InputStream aux1 = new URL(urlAddress).openStream();
      GZIPInputStream aux2 = new GZIPInputStream(aux1);
      InputStreamReader aux3 = new InputStreamReader(aux2);
      return new BufferedReader(aux3);
   
    } else {

      // the file is not compressed:

      InputStream aux1 = new URL(urlAddress).openStream();
      InputStreamReader aux2 = new InputStreamReader(aux1);
      return new BufferedReader(aux2);
    }
  }

  /////////////////////////////////////////////////////////////////////////////
  // This checks a string entry to see if it's a number.  If not, it produces
  // an error message and returns null.  It also checks to see if the number is
  // between lowerBound and upperBound.  If not, it produces a different error
  // message and returns null.
  /////////////////////////////////////////////////////////////////////////////
  public static Double readDouble(String str, String ParamName, double lowerBound,
                            double upperBound, JComponent c) {
    Double result;
    try {
      result = Double.parseDouble(str);
    } catch(Exception e) {
      JOptionPane.showMessageDialog(
        c, // parent component
        "Please enter a number for the "+ParamName,
        "Error",
        JOptionPane.ERROR_MESSAGE
      );
      return null;
    }
    if ((result > upperBound)||(result < lowerBound)) {
      JOptionPane.showMessageDialog(
        c, // parent component
        "The "+ParamName+" must be between "+((Double)lowerBound).toString()+
        " and "+((Double)upperBound).toString(),
        "Error",
        JOptionPane.ERROR_MESSAGE
      );
      return null;
    }
    return result;
  }

  /////////////////////////////////////////////////////////////////////////////
  // This checks a string entry to see if it's an integer.  If not, it produces
  // an error message and returns null.  It also checks to see if the integer
  // is between lowerBound and upperBound.  If not, it produces a different
  // error message and returns null.
  /////////////////////////////////////////////////////////////////////////////
  public static Integer readInt(String str, String ParamName, int lowerBound,
                            int upperBound, JComponent c) {
    Integer result;
    try {
      result = Integer.parseInt(str);
    } catch(Exception e) {
      JOptionPane.showMessageDialog(
        c, // parent component
        "Please enter an integer for the "+ParamName,
        "Error",
        JOptionPane.ERROR_MESSAGE
      );
      return null;
    }
    if ((result > upperBound)||(result < lowerBound)) {
      JOptionPane.showMessageDialog(
        c, // parent component
        "The "+ParamName+" must be between "+((Integer)lowerBound).toString()+
        " and "+((Integer)upperBound).toString(),
        "Error",
        JOptionPane.ERROR_MESSAGE
      );
      return null;
    }
    return result;
  }

}
